
{NetShell V1.00 writen by Mike Day 9-4-92}

Unit NetShell;

interface
uses TCPSYS;

type NetRec = record
       Nd : integer;     {net descriptor}
       Na : NetAddrType; {net address}
       Ne : byte;        {net error}
       Ns : byte;        {net sub-error}
       Nx : integer;     {temp function result}
       Wf : word;        {net write flags}
       Rf : word;        {net read flags}
     end;

var Net : NetRec;  {be sure to zero this before use}

{******************************************************************}
 {create net link resources}
function NetCreate(var Net:NetRec; var HostName:string;
                       BufSize,Timeout:longint; Port:word):integer;
function NetAttach(var Net:NetRec):integer;  {connect to the host}
function NetLook(var Rec:NetRec):integer;    {look for a connection request}

function NetRcvReady(var Net:NetRec):integer;  {any data to pick up?}
function NetSendReady(var Net:NetRec):integer; {can we send something?}
function NetRcv(var Net:NetRec; var Buf; BufSize:word):integer;  {get data}
function NetSend(var Net:NetRec; var Buf; BufSize:word):integer; {send data}
function NetClose(var Net:NetRec):integer;     {shut down the net link}


{Note: if an error occurs (function result is negative), you can}
{get the error numbers from Net.Ne and Net.Ns. See the TCPERR.INC}
{file for the meaning of the error numbers.}

{*******************************************************************}
implementation

procedure SaveError(var Net:NetRec);
begin
  Net.Ne := NetErrNo;
  Net.Ns := NetSubErrNo;
end;


{------------------------------------------------}
{create net link resources - Net is presumed to be unused}
{if Net was in use, a NetClose must be closed before calling this}
{Net contains the Net control record. HostName is the name of the host}
{you want to connect the Nd to. BufSize is the size of the buffer}
{to use with the descriptor. If BufSize is zero, then the default buffer}
{size will be used. Timeout is the length of time to wait on Listen or}
{connect befor giving up. Port is the port number to use to for the connection}
{Note: the TCP TSR gets the extra buffer space by allocating the memory}
{from regular memory with a standard DOS memory allocation call.}
{Your program must allow for enough free memory outside itself for the}
{allocation to succeed. If not, then this function will error out with}
{a not enough memory error.}
function NetCreate(var Net:NetRec; var HostName:string;
                       BufSize,Timeout:longint; Port:word):integer;
begin
  Net.Nd := -1;
  Net.Nx := NetGetDesc;
  if Net.Nx < 1 then
  begin
    NetCreate := Net.Nx;
    SaveError(Net);
    Exit;
  end;
  Net.Nd := Net.Nx;
  if BufSize > 0 then
    Net.Nx := SetOption(Net.Nd,0,NET_OPT_BUFSIZE,BufSize,Sizeof(Longint));
  if (Net.Nx > -1) and (TimeOut > 0) then
    Net.Nx := SetOption(Net.Nd,0,NET_OPT_TIMEOUT,TimeOut,Sizeof(Longint));
  if Net.Nx > -1 then
  begin
    HostName := HostName+#0;
    Net.Na.Fhost := NmResName(@HostName[1],nil,0);
    if Net.Na.Fhost = 0 then
    begin
      NetCreate := -1;
      SaveError(Net);
      Net.Nx := NetRelease(Net.Nd);
      Net.Nd := 0;
      Exit;
    end;
    Net.Na.Fsocket := port;
    Net.Na.Lsocket := 0;
    Net.Na.protocol := STREAM;
    Net.Rf := NET_FLG_NONBLOCKING;
    Net.Wf := NET_FLG_NONBLOCKING;
    Net.Nx := 0;
  end;
  NetCreate := Net.Nx;
end;

{------------------------------------------------}
{This attaches the descriptor to the host system previously }
{set up by NetCreate as "Connect" (client)}
{if the connect fails, you can try again without doing}
{another NetCreate. In fact, If you want to do a NetCreate}
{on this descriptor, you must call a NetRelease to release}
{it first.}
function NetAttach(var Net:NetRec):integer;
begin
  Net.Nx := NetConnect(Net.Nd,Net.Na.Protocol,@Net.Na);
  if Net.Nx < 0 then
    SaveError(Net);
  NetAttach := Net.Nx;
end;

{------------------------------------------------}
{This attaches the descriptor to the client previously }
{set up by NetCreate as "Listen" (server)}
{if the NetLook fails, you can try again without doing}
{another NetCreate. In fact, If you want to do a NetCreate}
{on this descriptor, you must call a NetRelease to release}
{it first. NetLook will wait for a connection request by the}
{time that was speced in the TimeOut value in NetCreate.}
function NetLook(var Rec:NetRec):integer;    {look for a connection request}
begin
  Net.Nx := NetListen(Net.Nd,Net.Na.Protocol,@Net.Na);
  if Net.Nx < 0 then
    SaveError(Net);
  NetLook := Net.Nx;
end;

{------------------------------------------------}
{check if there is any data waiting in the recv buffer}
{returns count of bytes waiting to be read if any}
{returns zero if nothing waiting. }
{Returns a negative number if something went wrong}
{See the Net.Ne (net error) and Net.Ns (net suberror) }
{for the reason of the failure. See TCPERR.INC for errors}
{Note: this presumes that "NET" has already been setup}
function NetRcvReady(var Net:NetRec):integer;
begin
  Net.Nx := NetRead(Net.Nd,nil,0,@Net.Na,NET_FLG_PEEK or NET_FLG_NONBLOCKING);
  NetRcvReady := Net.Nx;
  if Net.Nx < 0 then
    SaveError(Net);
end;

{------------------------------------------------}
{check if the write buffer is available to send data}
{returns count of bytes available in the buffer if free}
{returns zero if buffer is full. }
{Returns a negative number if something went wrong}
{See the Net.Ne (net error) and Net.Ns (net suberror) }
{for the reason of the failure. See TCPERR.INC for errors}
{Note: this presumes that "NET" has already been setup}
function NetSendReady(var Net:NetRec):integer;
begin
  writeln ('*** NetSendReady not implemented yet ***');
  NetSendReady := 0;
end;

{------------------------------------------------}
{Pick up the data waiting to be read into Buf }
{The BufSize must be equal to or larger than the data }
{waiting in the Net buffer. The data count was returned}
{by the NetRcvReady routine.}
{returns count of bytes that were read if any}
{returns zero if nothing to be read. }
{Returns a negative number if something went wrong}
{See the Net.Ne (net error) and Net.Ns (net suberror) }
{for the reason of the failure. See TCPERR.INC for errors}
{Note: this presumes that "NET" has already been setup}
function NetRcv(var Net:NetRec; var Buf; BufSize:word):integer;
begin
  Net.Nx := NetRead(Net.Nd,@Buf,BufSize,@Net.Na,Net.Rf);
  NetRcv := Net.Nx;
  if Net.Nx < 0 then
    SaveError(Net);
end;

{------------------------------------------------}
{This sends the data in BUF out the network port}
{returns count of bytes that were sent if any}
{returns zero if nothing was sent. }
{Returns a negative number if something went wrong}
{See the Net.Ne (net error) and Net.Ns (net suberror) }
{for the reason of the failure. See TCPERR.INC for errors}
{Note: this presumes that "NET" has already been setup}
function NetSend(var Net:NetRec; var Buf; BufSize:word):integer;
begin
  Net.Nx := NetWrite(Net.Nd,@Buf,BufSize,Net.Wf);
  NetSend := Net.Nx;
  if Net.Nx > -1 then
    Net.Nx := NetFlush(Net.Nd);
  if Net.Nx < 0 then
  begin
    NetSend := Net.Nx;
    SaveError(Net);
  end;
end;

{------------------------------------------------}
{This shuts down the net link}
{Note: this presumes that "NET" has already been setup}
function NetClose(var Net:NetRec):integer;
begin
   Net.Nx := NetEOF(Net.Nd);
   if Net.Nx > -1 then
     Net.Nx := NetFlush(Net.Nd);
   if Net.Nx > -1 then
     Net.Nx := NetRelease(Net.Nd);
   if Net.Nx < 0 then
   begin
     NetClose := Net.Nx;
     SaveError(Net);
     Net.Nx := NetAbort(Net.Nd);
     Exit;
   end;
   NetClose := 0;
end;

{-------------------------------}
{init}
begin
  fillchar(Net,sizeof(NetRec),0);
end.
