unit TcpSys;   {Version 1.10  10/05/93  -med}

{ TCP/IP System Call interface unit for Turbo Pascal V7.0 }
{ Copyright 1992,1993 Knight Software }
{ First release: 3 Jul 1992 - Mike Day }
{ 2nd release:  23 Jul 1992 - Mike Day }
{ 3rd release:  31 Aug 1992 - Mike Day fixed param passing}
{ 4th release:   2 Sep 1992 - Mike Day moved byte ordering into here}
{ 5 Feb 1992 - Mike Day : no longer halt if Tcp tsr not installed}
{                         now check the TcpInstalled flag before using}
{                         fix check break problem causing blow up}
{ 17 Aug 1993 - Mike Day : reviewed for protected mode operation}
{                          massive conversion performed to do pmode}
{                          see NETCOPY note for read/write operation}
{ 5 Oct 1993 - Mike Day : general cleanup}

interface
{$IFDEF WINDOWS} {$DEFINE PMODE} {$ENDIF}
{$IFDEF DPMI}    {$DEFINE PMODE} {$ENDIF}

{$IFDEF PMODE}
  uses WinApi;
 {$DEFINE NETCOPY} { <--see note below--} {NETCOPY is for pmode use only}
{$ENDIF}

{---------------------------------------------------------------------------}
{With NETCOPY defined, TCPSYS is completely transparent to the application}
{you can use any memory you want. The down side is that you are limited to}
{a buffer size of NetDataSize for reads and writes because the read write}
{routines will copy the data to a DOS buffer to be compatible with the TCP}
{network TSR. This of course will slow things down a bit because of the}
{need to copy the data back and forth. If you undefine NETCOPY, then you}
{*must* pass the buffer pointer in NetRead and NetWrite as a selector}
{that was allocated with GlobalDosAlloc. The advantage to this is a }
{slight speed improvment since the copying is not required. The disadvantage}
{is that it requires a different call format, and it is possible for the}
{allocation to fail during run time due to lack of dos memory.}
{---------------------------------------------------------------------------}

const PcTcpInt : byte = 0;
      OldDosBreak : byte = 0;
      TcpInstalled : boolean = false; {is Tcp Tsr installed?}
      PmodePcTcpInt : byte = $61;   {In pmode, we don't search for the int}
                                    {we just presume it is here}

{***************************************************************}
{TCP constants}

{Option values for SetOption and GetOption system calls}
Const NET_OPT_NONBLOCKING = 1;
      NET_OPT_TIMEOUT     = 2;
      NET_OPT_HINT        = 3;
      NET_OPT_BUFSIZE     = 4;
      NET_OPT_NETASCII    = 5;
      NET_OPT_KEEPALIVE   = 6;
      NET_OPT_DEBUG       = 7;
      NET_OPT_DONTROUTE   = 8;
      NET_OPT_PRECEDENCE  = 9;
      NET_OPT_TOS         = 10;
      NET_OPT_PRIV        = 11;
      NET_OPT_WAITFLUSH   = 12;

{----------------------------------------------------------------}
{Service Connection types associated with net descriptors used in}
{NetConnect and NetListen}
Const RAW_NET  = 1;  {Note: FTP says they have not implemented RAW_NET}
      RAW_IP   = 2;
      DGRAM    = 3;
      STREAM   = 4;
      RAW_ICMP = 5;

{------------------------------------------------------}
{Flags for: NetRead, NetReadFrom, NetRecv, NetRecvFrom,}
{           NetWrite, NetWriteTo, NetSend, NetSendTo }
Const NET_FLG_OOB          = $0001; {out of band data}
      NET_FLG_PEEK         = $0002; {don't dequeue data}
      NET_FLG_DRAIN        = $0004; {don't copy data}
      NET_FLG_DONTROUTE    = $0008; {reserved}
      NET_FLG_PUSH         = $0010; {reserved}
      NET_FLG_DONTTRUNCATE = $0020; {Don't truncate oversized dgram}
      NET_FLG_NONBLOCKING  = $0040; {don't block}
      NET_FLG_BROADCAST    = $0080; {broadcast data}

{--------------------------------------------------------------------}
{misc type defs used by TCP/IP}
Type InName = longint;

{--------------------------------------------------------------------}
{network address structure for NetConnect() and NetListen() for types}
{ RAW_IP, DGRAM and STREAM (structure is called "addr" in socket code)}
Type  NetAddrType = record
        fhost   : InName; { holds Internet address in some byte order }
        fsocket : word;   { holds "foreign port" number }
        lsocket : word;   { holds "local port" number }
        protocol: byte;   { should be "STREAM" for Lidar stuff }
      end;
      NetAddrPtr = ^NetAddrType;
const NetAddrSize = sizeof(NetAddrType);

{----------------------------------------------------------------------}
{Network address record structure for NetConnect and NetListen }
{for type RAW_NET. (see INTERFAC.INC) }
{*** Warning: FTP says they have not implemented RAW_NET ***}
{See the TCP/IP development kit read.me file for more info}
Type RawNetAddr = record
       NetClass : integer;
       NetType  : integer;
       Number   : integer;
     end;

{----------------------------------------------------------------------}
{Record used to return info about network - Used by netInfo }
{Warning! LanAddr returns a real mode Seg:Ofs pointer even in Pmode}
{you must convert it to a selector if you want to access the data}
{it is pointing at. Remember to release the selector when you are done}
Type  NetInfoRec = record
        IsClass   : integer; {The class of the interface 802.3, 802.5}
        IsType    : integer; {The type of the interface: manufacturer}
        Number    : integer; {Interface number (0,1..) of this class/type}
        IpAddress : InName;  {This interface's IP address}
        IpSubnet  : InName;  {This interface's subnet and network mask}
        IsUp      : integer; {NZ if this interface is up}
        PacketIn  : longint; {number packets received}
        PacketOut : longint; {number packets sent}
        InErr     : longint; {number receiver errors}
        OutErr    : longint; {number transmitter errors}
        LanLen    : integer; {length of local net address}
       {$IFDEF PMODE}
        LanAddr   : record RealOfs,RealSeg:word; end;
       {$ELSE}
        LanAddr   : pointer; {real mode Pointer to the LAN address}
       {$ENDIF}
      end;
      NetInfoPtr = ^NetInfoRec;
const NetInfoSize = sizeof(NetInfoRec);

{----------------------------------------------------------------------}
{Record used to return info about network - Used by netInfo }
Type  NetStatsRec = record
        BytesUnxmt     : longint; {bytes left untransmitted}
        BytesUnRcv     : longint; {bytes left unreceived}
        OobBytesUnxmt  : longint; {oob bytes left untransmitted}
        OobBytesUnRcv  : longint; {oob bytes left unreceived}
        BytesXmt       : longint; {bytes transmitted}
        BytesRcv       : longint; {bytes received}
        OobBytesXmt    : longint; {oob bytes transmitted}
        OobBytesRcv    : longint; {oob bytes received}
        PktsXmt        : longint; {packets transmitted}
        PktsRcv        : longint; {packets received}
        BadXsums       : longint; {bad checksums}
        ProtErrors     : longint; {protection errors}
        Timeouts       : longint; {number timeouts}
        Errors         : longint; {number errors}
        RcvRetransmits : longint; {number receive retransmits}
        XmtRetransmits : longint; {number transmit retransmits}
      end;
      NetStatsPtr = ^NetStatsRec;
const NetStatsSize = sizeof(NetStatsRec);

{----------------------------------------------------------------------}
{record used to return info about kernel - Used by NetGetKconf }
Type  KernelConfRec = record
        KmaxTcp:byte;      {# of TCP connections allowed}
        KmaxUdp:byte;      {# of UDP connections allowed}
        KmaxIp :byte;      {# of IP connections allowed}
        KmaxRaw:byte;      {# of RAW_NET connections allowed}
        KactTcp:byte;      {# of TCP connections in use}
        KactUdp:byte;      {# of UDP connections in use}
        KactIp :byte;      {# of IP connections in use}
        KactRaw:byte;      {# of RAW_NET connections in use}
        KactLcl:word;      {# of active Local network descriptors}
        KactGbl:word;      {# of active Global network descriptors}
        KMaxLnh:byte;      { Maximum header on an attached network}
        KMaxLnt:byte;      { Maximum trailer on an attached network}
        KLBUF  :word;      { Size of a kernel packet buffer}
        KNnet  :word;      { Number of network interfaces attached}
        Kcticks:longint;   { Milliseconds since kernel started}
        Kbroadcast:InName; { IP broadcast address in use}
      end;
      KernelConfPtr = ^KernelConfRec;
const KernelConfSize = sizeof(KernelConfRec);

{----------------------------------------------------------------}
{misc variables used by TCP/IP}
var NetErrNo    : byte;
    NetSubErrNo : byte;
    PatchLevel  : word;
    RetFlags    : word;


{===============================================================}
(*
  Notation: {R} = Real mode only function, not avail in prot mode
            {P} = Real or protected mode, same functionality
            {N} = Special case for protected mode operation
*)

{***************************************************************}
{Network Descriptor Calls}
function GetAddr(nd:integer):InName;                                     {P}
function GetOption(nd,lev,opt:integer; p:pointer; len:integer):longint;  {P}
function GetPeer(nd:integer; nap:NetAddrPtr):integer;                    {P}
function NetAbort(nd:integer):integer;                                   {P}
function NetAbortAll:integer;                                            {P}
function NetConnect(nd,ts:integer; nap:NetAddrPtr):integer;              {P}
function NetEof(nd:integer):integer;                                     {P}
function NetFlush(nd:integer):integer;                                   {P}
function NetGetDesc:integer;                                             {P}
function NetGetGlobDesc:integer;                                         {P}
function NetGlobalize(nd:integer):integer;                               {P}
function NetListen(nd,ts:integer; nap:NetAddrPtr):integer;               {P}
function NetRelease(nd:integer):integer;                                 {P}
function NetReleaseAll:integer;                                          {P}
function NetSwap(Nd1,Nd2:integer):integer;                               {P}
function SetOption(nd,lev,opt:integer; ov:longint; len:integer):integer;

{***************************************************************}
{Transmit Calls}

{$IFDEF PMODE} {$IFNDEF NETCOPY}
 {$DEFINE NETDIRECT}
{$ENDIF} {$ENDIF}

{$IFNDEF NETPDIRECT}
 function NetWrite(nd:integer; buf:pointer; len:word; flags:word):integer;{N}
 function NetWriteTo(nd:integer; buf:pointer; len:word;
                     toaddr:pointer; Flags:word):integer;                 {N}
{$ELSE}
 function NetWrite(nd:integer; buf:longint; len:word; flags:word):integer;{N}
 function NetWriteTo(nd:integer; buf:longint; len:word;
                     toaddr:pointer; Flags:word):integer;                 {N}
{$ENDIF}

{***************************************************************}
{Receive Calls}
{$IFNDEF NETPDIRECT}
 function NetRead(nd:integer; buf:pointer; len:word;
                  fromaddr:pointer; Flags:word):integer;            {N}
 function NetReadFrom(nd:integer; buf:pointer; len:word;
                     fromaddr:pointer; Flags:word):integer;         {N}
{$ELSE}
 function NetRead(nd:integer; buf:longint; len:word;                {N}
                  fromaddr:pointer; Flags:word):integer;
 function NetReadFrom(nd:integer; buf:longint; len:word;            {N}
                     fromaddr:pointer; Flags:word):integer;
{$ENDIF}

{***************************************************************}
{Status Calls}
function IsNetnd(nd:integer):integer;                               {P}
function NetInfo(nd:integer; ni:NetInfoPtr):integer;                {P}
function NetSelect(maxid:integer; iflags,oflags:pointer):integer;   {P}
function NetStat(nd:integer; stats:NetStatsPtr):integer;            {P}

{***************************************************************}
{Asynchronous Communications Calls}       {<--this stuff is not }
{$IFNDEF PMODE}                           {<--available in      }
 function DisableAsynch:integer;          {<--protected mode    }   {R}
 function EnableAsynch:integer;                                     {R}
 function NetAlarm(fd:integer; Atime:longint;                       {R}
                   handler:pointer; hint:longint):integer;
 function NetAsynch(fd,event:integer;                               {R}
                   handler:pointer; hint:longint):pointer;
{$ENDIF}

{***************************************************************}
{Host Name Resolution Calls}
function AdDomain(IPaddr:InName; name:pointer; len:word):integer;   {P}
function AdHtable(IPaddr:InName; name:pointer; len:word):integer;   {P}
function AdResName(IPaddr:InName; name:pointer; len:word):integer;  {P}
function NmDomain(name,cname:pointer; len:integer):InName;          {P}
function NmHtable(name,cname:pointer; len:integer):InName;          {P}
function NmIen116(name:pointer):InName;                             {P}
function NmPrsAddr(name:pointer):InName;                            {P}
function NmResName(name,cname:pointer; len:integer):InName;         {P}

{***************************************************************}
{ICMP Calls}
function IcmpPing(Host:InName; Len:integer):integer;                {P}

{***************************************************************}
{Miscellaneous}
function  GetNetVersion:word;                                       {P}
procedure NetReconfig;                                              {P}
procedure NetShutDown;                                              {P}
function  NetGetKConf(kc:KernelConfPtr):integer;                    {P}

{***************************************************************}
{Byte Ordering}
function Htonl(L:longint):longint; {network longint to PC Longint}  {P}
function Htons(W:word):word;       {network word to PC word}        {P}
function Ntohl(L:longint):longint; {PC longint to Network Longint}  {P}
function Ntohs(W:word):word;       {PC word to Network word}        {P}


{===============================================================}
implementation

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{Special local procedures}

  {TCP/IP interrupt number: 0=search for the tsr; Note: $61 is default int}
  const TCPinterrupt = 0;

{$IFDEF PMODE}
  const Nnap  : longint = 0;  {pmode segment/selector for temp nap}
        Nni   : longint = 0;  {pmode segment/selector for temp ni}
        Nstats: longint = 0;  {pmode segment/selector for temp stats}
        Nkc   : longint = 0;  {pmode segment/selector for temp kc}
        Nname : longint = 0;  {pmode segment/selector for temp name}
        Ncname: longint = 0;  {pmode segment/selector for temp cname}
        Ntemp : longint = 0;  {pmode segment/selector for temp var}
        NetData:longint = 0;  {pmode segment/selector for temp data}

  const NnameSize = sizeof(string)-1;
        NtempSize = 16;
        NetDataSize = 16384;

  const TCPintNum : byte = TCPinterrupt;
  var SimInt : record
        RealDI : longint;   {register structure used for    }
        RealSI : longint;   {processing real mode interrupts}
        RealBP : longint;   {from protected mode operation  }
        RealXX : longint;
        RealBX : longint;
        RealDX : longint;
        RealCX : longint;
        RealAX : longint;
        RealFlags : word;
        RealES : word;
        RealDS : word;
        RealFS : word;
        RealGS : word;
        RealIP : word;
        RealCS : word;
        RealSP : word;
        RealSS : word;
      end;

  {this function presumes the SimInt registers have been properly set}
  {before the call is made. updated registers are returned in SimInt}
  {this must be assembler. Do NOT pass any parameters.}
  function TCPsysInt:boolean; assembler;
  asm
    MOV word ptr [SimInt.RealAX],AX
    MOV word ptr [SimInt.RealBX],BX   {pass std regs}
    MOV word ptr [SimInt.RealCX],CX
    MOV word ptr [SimInt.RealDX],DX
    MOV word ptr [SimInt.RealSS],0    {let DPMI make it's own stack}
    MOV word ptr [SimInt.RealSP],0
    MOV word ptr [SimInt.RealXX],0
    MOV word ptr [SimInt.RealXX+2],0
    LEA DI,SimInt
    MOV AX,$0300
    MOV BL,[TCPintNum]
    OR BL,BL           {int 0 is not valid, must not be set}
    JZ @DpmiFail
    MOV BH,0
    MOV CX,0
    INT $31      {call the dpmi to do the interrupt}
    JC @DpmiFail
    MOV AH,byte ptr [SimInt.RealFlags] {Joy! call was successful}
    SAHF
    MOV AX,word ptr [SimInt.RealAX]
    JMP @DpmiRet
   @DpmiFail:
    MOV AX,-1    {eek! dpmi call failed}
    STC
   @DpmiRet:
    MOV BX,word ptr [SimInt.RealBX]   {restore std regs}
    MOV CX,word ptr [SimInt.RealCX]
    MOV DX,word ptr [SimInt.RealDX]
  end;

  procedure SetTcpVector(v:byte);
  begin
    TcpIntNum := v;
  end;
  procedure ZapTcpTsrCall;
  begin
    TcpIntNum := 0;
  end;
{$ELSE}

  {Warning: This must be assembler, and it must stay exactly like this}
  {SetTCPvector modifies this to point at the real TCP interrupt.}
  procedure TCPsysInt; near; assembler;  { *must* be near }
  asm
    int TCPinterrupt
  end;

  {modify TCPsysInt to point at the specified interrupt}
  {self modifying code...yuck}
  procedure SetTCPvector(v:byte);
  type tcpintproccode = array[0..1] of byte;
  var p:^tcpintproccode;
  begin
    p := @TCPsysInt;
    p^[1] := v;
  end;

  {modify TCPsysInt to do nothing}
  {self modifying code...yuck}
  procedure ZapTcpTsrCall;
  type tcpintproccode = array[0..1] of byte;
  var p:^tcpintproccode;
  begin
    p := @TCPsysInt;
    p^[0] := $90;  {nop}  {nop out the int call}
    p^[1] := $90;  {nop}
  end;

{$ENDIF}

{***************************************************************}
{Network Descriptor Calls}

{GetAddr is called with the net descriptor in 'nd'}
{returns the address in the function result.}
{on error, returns zero with the encountered error saved in NetErr}
{Possible errors are: ILLEGALOP, NOTNETCONN, UNBOUND}
function GetAddr(nd:integer):InName; assembler; {ok}
asm
    push ds
    mov ah,5
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @done
    xor ax,ax
    xor dx,dx
   @done:
end;

{---------------------------------------------------------------}
{GetOption is called with the net descriptor in 'nd'}
{'Lev' is currently ignored, Opt contains the option to get}
{See constants in interface for option numbers allowed}
{Returns the option result in the array pointed at by P}
{Also returns option result in the function return.}
{Possible errors are: BADARG, NOTNETCONN }
function GetOption(nd,lev,opt:integer; p:pointer; len:integer):longint; assembler;
asm
    push ds
    mov ah,$21
    mov bx,[nd]
   {$IFDEF PMODE}
    mov di,[opt]
    mov word ptr [SimInt.RealDI],DI
    mov si,[lev]
    mov word ptr [SimInt.RealSI],SI
    mov dx,word ptr [Ntemp+2]
    MOV word ptr [SimInt.RealDS],DX {P}
    MOV word ptr [SimInt.RealDX],0
   {$ELSE}
    mov si,[lev]
    mov di,[opt]
    lds dx,[p]
   {$ENDIF}
    mov cx,[len]
    CALL TCPsysInt
    mov cx,ax
    mov ax,dx
   {$IFDEF PMODE}
    mov dx,word ptr [SimInt.RealDS]
   {$ELSE}
    mov dx,ds
   {$ENDIF}
    pop ds
    jnc @done
    mov [neterrno],cl
    mov [netsuberrno],ch
    mov ax,cx
    cbw          {is this stuff right?}
    xor ax,-1
    xor dx,ax
   @done:
  {$IFDEF PMODE}
    mov cx,[len]
    cmp cx,0
    jz @nobuf
    les di,[p]             {es:di points to dest (p^)}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0
    MOV DS,word ptr [Ntemp] {ds:si points at src (Dtemp^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   @nobuf:
  {$ENDIF}
end;

{-------------------------------------------------}
{GetPeer is called with the net descriptor in 'nd'}
{Returns the peer address in the record pointed at by nap}
{Possible errors are: NOTNETCONN, UNBOUND, ILLEGALOP }
function GetPeer(nd:integer; nap:NetAddrPtr):integer; assembler; {ok}
  asm
    push ds
    mov ah,$16
    mov bx,[nd]
   {$IFDEF PMODE}
    MOV DX,word ptr [Nnap+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
   {$ELSE}
    lds dx,[nap]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {result returned in AX}
   @done:
  {$IFDEF PMODE}
    les di,[nap]           {es:di points to dest (nap^)}
    MOV CX,NetAddrSize     {get size of record}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0
    MOV DS,word ptr [Nnap] {ds:si points at src (Dnap^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
  {$ENDIF}
  end;

{--------------------------------------------------}
{NetAbort is called with the net descriptor in 'nd'}
{Possible errors are: NOTNETCONN }
function NetAbort(nd:integer):integer; assembler; {ok}
asm
    push ds
    mov ah,$19
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @naok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @naok:
    xor ax,ax
   @done:
end;

{--------------------------------------------}
{NetAbortAll does NetAbort on all descriptors}
{no error is returned from this function}
function NetAbortAll:integer; assembler;  {ok}
asm
    push ds
    mov ah,$24
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-----------------------------------------------------------}
{NetConnect opens a net connection. If nd has a number, that}
{net descriptor (handle) will be used. If nd = -1, NetConnect}
{will allocate a descriptor and return the new descriptor in}
{the function result. ts contains the service type}
{nap points to the net address record for the descriptor}
{Possible errors are: BADARG, INUSE, NOHOST, NODESC, NOMEM}
{                     NOTNETCONN, RESET, TIMEOUT}
function NetConnect(nd,ts:integer; nap:NetAddrPtr):integer; assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NetAddrSize     {get size of record}
    mov es,word ptr [Nnap] {es:di points to dest (Dnap^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [nap]
    MOV DS,word ptr [nap+2] {ds:si points at src (nap^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$13
    mov bx,[nd]
    mov dx,[ts]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nnap+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[nap]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {result returned in AX}
   @done:
   {$IFDEF PMODE}
    les di,[nap]           {es:di points to dest (nap^)}
    MOV CX,NetAddrSize     {get size of record}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0
    MOV DS,word ptr [Nnap] {es:di points at src (Dnap^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{NetEof is called with the net descriptor in 'nd'}
{Requests that the transmitter specified by 'nd' be shut down}
{Possible errors are: NOTNETCONN, ILLEGALOP, NOTESTAB }
function NetEof(nd:integer):integer; assembler; {ok}
asm
    push ds
    mov ah,$18
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @retok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @retok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{NetFlush is called with the net descriptor in 'nd'}
{Requests that the nd buffer be flushed immediately}
{Possible errors are: NOTNETCONN, ILLEGALOP, REENTRY }
function NetFlush(nd:integer):integer; assembler;  {ok}
asm
    push ds
    mov ah,$1E
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @nfok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nfok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{NetGetDesc returns the local descriptor}
{Possible errors are: NET_DOS_ERROR, NET_ERR_NODESC, NET_ERR_NOMEM}
function NetGetDesc:integer; assembler;  {ok}
asm
    push ds
    mov ah,$22
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-------------------------------------------------}
{NetGetDesc returns a global network descriptor}
{Possible errors are: NODESC, NOMEM}
function NetGetGlobDesc:integer; assembler; {ok}
asm
    push ds
    mov ah,$29
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-------------------------------------------------}
{NetGlobalize converts a network descriptor to a global descriptor}
{returns the global descriptor in the function result}
{Possible errors are: NODESC, NOTNETCONN}
function NetGlobalize(nd:integer):integer; assembler; {ok}
asm
    push ds
    mov ah,$07
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-------------------------------------------------}
{NetListen opens a net connection for listening. If nd has a number,}
{that net descriptor (handle) will be used. If nd = -1, NetListen}
{will allocate a descriptor and return the new descriptor in}
{the function result. ts contains the service type}
{nap points to the net address record for the descriptor}
{Possible errors are: BADARG, INUSE, NODESC, NOMEM}
{                     NOTNETCONN, TIMEOUT, UNBOUND}
function NetListen(nd,ts:integer; nap:NetAddrPtr):integer; Assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NetAddrSize     {get size of record}
    mov es,word ptr [Nnap] {es:di points to dest (Dnap^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [nap]
    MOV DS,word ptr [nap+2] {ds:si points at src (nap^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$23
    mov bx,[nd]
    mov dx,[ts]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nnap+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[nap]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {result returned in AX}
   @done:
   {$IFDEF PMODE}
    les di,[nap]           {es:di points to dest (nap^)}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV CX,NetAddrSize     {get size of record}
    MOV SI,0
    MOV DS,word ptr [Nnap] {ds:si points to src (Dnap^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{NetRelease releases the network connection for}
{the descriptor passed in nd}
{Possible errors are: NOTNETCONN, TIMEOUT }
function NetRelease(nd:integer):integer; assembler;  {ok}
asm
    push ds
    mov ah,$08
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @nrelok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nrelok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{NetReleaseAll releases all network descriptors.}
{Note: does not release global descriptors}
function NetReleaseAll:integer; assembler; {ok}
asm
    push ds
    mov ah,$09
    CALL TCPsysInt
    pop ds
    jnc @nrelaok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nrelaok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{NetSwap swaps net connections between the two descriptors}
{Possible errors are: NOTNETCONN }
function NetSwap(Nd1,Nd2:integer):integer; assembler; {ok}
asm
    push ds
    mov ah,$28
    mov bx,[nd1]
    mov cx,[nd2]
    CALL TCPsysInt
    pop ds
    jnc @nswpok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nswpok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{SetOption sets the desired option for the specified net descriptor in nd}
{'Lev' is currently ignored, Opt contains the option to set}
{See constants in interface for option numbers allowed}
{option value is passed in op. }
{len must indicate the length of the value: 0=boolean, 2=integer, 4=longint}
{Possible errors are: BADARG, NOTNETCONN }
function SetOption(nd,lev,opt:integer; ov:longint; len:integer):integer; assembler; {ok}
asm
    push ds
    mov ah,$20
    mov bx,[nd]
   {$IFDEF PMODE}
    mov si,[lev]
    mov word ptr [SimInt.RealSI],SI
    mov di,[opt]
    mov word ptr [SimInt.RealDI],DI
    mov dx,word ptr [ov+2]
    MOV word ptr [SimInt.RealDS],DX
    mov dx,word ptr [ov]
    MOV word ptr [SimInt.RealDX],DX
   {$ELSE}
    mov si,[lev]
    mov di,[opt]
    lds dx,[ov]
   {$ENDIF}
    mov cx,[len]
    CALL TCPsysInt
    pop ds
    jnc @nsook
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nsook:
    xor ax,ax
   @done:
end;


{***************************************************************}
{Transmit Calls}

{-------------------------------------------------}
{NetWrite writes the specified data to send to the descriptor}
{nd=descriptor, buf=points to data buffer to send,}
{len=length of data in bytes, flags=options to use on transmission}
{See NetWrite flag constants above for options to use}
{Returns data length or error number, the returned flags}
{are stored in the RetFlags variable.}
{*** NOTE: *** See notes in title about NETCOPY ***}
{Possible errors are: EOF, NOHOST, NOTESTAB, NOTNETCONN }
{if PMODE and not NETCOPY, buf is the longint result of a GlobalDosAlloc}

{$IFNDEF NETPDIRECT}
 function NetWrite(nd:integer; buf:pointer; len:word; flags:word):integer; assembler;
{$ELSE}
 function NetWrite(nd:integer; buf:longint; len:word; flags:word):integer; assembler;
{$ENDIF}
asm
  {$IFDEF PMODE}
   {$IFDEF NETPDIRECT}
    MOV CX,[len]              {get size of record}
    mov es,word ptr [NetData] {es:di points to dest (NetData^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [buf]
    MOV DS,word ptr [buf+2] {ds:si points at src (buf^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
    mov SI,word ptr [NetData+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ELSE}
    mov SI,word ptr [buf+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ENDIF}
  {$ENDIF}
    push ds
    mov ah,$1A
    mov bx,[nd]
    mov cx,[len]
    mov dx,[flags]
   {$IFNDEF PMODE}
    lds si,[buf]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    mov [RetFlags],dx
    jnc @done
   @ErrExit:
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-------------------------------------------------}
{NetWriteTo writes the specified data to send to the descriptor}
{nd=descriptor, buf=points to data buffer to send,}
{len=length of data in bytes, to=pointer to addr record for host}
{to be writen to, flags=options to use on transmission}
{See NetWriteTo flag constants above for options to use}
{Returns data length or error number, the returned flags}
{are stored in the RetFlags variable.}
{*** NOTE: *** See notes in title about NETCOPY ***}
{Possible errors are: EOF, ILLEGALOP, NOHOST, NOTNETCONN }

{$IFNDEF NETPDIRECT}
 function NetWriteTo(nd:integer; buf:pointer; len:word;
                     toaddr:pointer; Flags:word):integer; assembler;
{$ELSE}
 function NetWriteTo(nd:integer; buf:longint; len:word;
                     toaddr:pointer; Flags:word):integer; assembler;
{$ENDIF}
asm
  {$IFDEF PMODE}
    MOV CX,NetAddrSize     {get size of record}
    mov es,word ptr [Nnap] {es:di points to dest (Dnap^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [toaddr]
    MOV DS,word ptr [toaddr+2] {ds:si points at src (toaddr^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
    mov DI,word ptr [Nnap]
    mov word ptr [SimInt.RealES],DI
    mov word ptr [SimInt.RealDI],0
   {$IFDEF NETPDIRECT}
    MOV CX,[len]              {get size of record}
    mov es,word ptr [NetData] {es:di points to dest (NetData^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [buf]
    MOV DS,word ptr [buf+2] {ds:si points at src (buf^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
    mov SI,word ptr [NetData+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ELSE}
    mov SI,word ptr [buf+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ENDIF}
  {$ENDIF}
    push ds
    mov ah,$1C
    mov bx,[nd]
    mov cx,[len]
    mov dx,[flags]
   {$IFNDEF PMODE}
    lds si,[buf]
    les di,[toaddr]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    mov [RetFlags],dx
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;


{***************************************************************}
{Receive Calls}


{-------------------------------------------------}
{NetRead reads the specified data to send to the descriptor}
{nd=descriptor, buf=points to data buffer to send,}
{len=length of data in bytes, flags=options to use on transmission}
{from=pointer to addr to be filled with from connection info}
{See NetWrite flag constants above for options to use}
{Returns data length or error number, the returned flags}
{are stored in the RetFlags variable.}
{*** NOTE: *** See notes in title about NETCOPY ***}
{Possible errors are: EOF, ICMPMSG, NOTESTAB, NOTNETCONN,}
{                     TIMEOUT, UNBOUND, WOULDBLOCK }

{$IFNDEF NETPDIRECT}
 function NetRead(nd:integer; buf:pointer; len:word;
                  fromaddr:pointer; Flags:word):integer; assembler;
{$ELSE}
 function NetRead(nd:integer; buf:longint; len:word;
                  fromaddr:pointer; Flags:word):integer; assembler;
{$ENDIF}
asm
  {$IFDEF PMODE}
    MOV CX,NetAddrSize     {get size of record}
    mov es,word ptr [Nnap] {es:di points to dest (Dnap^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [fromaddr]
    MOV DS,word ptr [fromaddr+2] {ds:si points at src (fromaddr^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
    mov DI,word ptr [Nnap]
    mov word ptr [SimInt.RealES],DI
    mov word ptr [SimInt.RealDI],0
   {$IFDEF NETPDIRECT}
    mov SI,word ptr [NetData+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ELSE}
    mov SI,word ptr [buf+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ENDIF}
  {$ENDIF}
    push ds
    mov ah,$1B
    mov bx,[nd]
    mov cx,[len]
    mov dx,[flags]
   {$IFNDEF PMODE}
    lds si,[buf]
    les di,[fromaddr]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    mov [RetFlags],dx
    jc @retbad
    mov ax,cx
   {$IFDEF PMODE} {$IFDEF NETPDIRECT}
    LES DI,[buf]              {es:di points at dest (buf^)}
    CLD                       {make sure it is in the right direction}
    PUSH DS                   {save ds because we're gonna use it}
    mov ds,word ptr [NetData] {ds:si points to src (NetData^)}
    mov si,0                  {reg cx still has length}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF} {$ENDIF}
    jmp @done
   @retbad:
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{-------------------------------------------------}
{NetReadFrom reads the specified data to send to the descriptor}
{nd=descriptor, buf=points to data buffer to send,}
{len=length of data in bytes, flags=options to use on transmission}
{from=pointer to addr of host to receive data from}
{See NetWrite flag constants above for options to use}
{Returns data length or error number, the returned flags}
{are stored in the RetFlags variable.}
{*** NOTE: *** See notes in title about NETCOPY ***}
{Possible errors are: BADARG, ILLEGALOP, NOTNETCONN, }
{                     UNBOUND, WOULDBLOCK }

{$IFNDEF NETPDIRECT}
 function NetReadFrom(nd:integer; buf:pointer; len:word;
                     fromaddr:pointer; Flags:word):integer; assembler;
{$ELSE}
 function NetReadFrom(nd:integer; buf:longint; len:word;
                     fromaddr:pointer; Flags:word):integer; assembler;
{$ENDIF}
asm
  {$IFDEF PMODE}
    MOV CX,NetAddrSize     {get size of record}
    mov es,word ptr [Nnap] {es:di points to dest (Dnap^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [fromaddr]
    MOV DS,word ptr [fromaddr+2] {ds:si points at src (fromaddr^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
    mov DI,word ptr [Nnap]
    mov word ptr [SimInt.RealES],DI
    mov word ptr [SimInt.RealDI],0
   {$IFDEF NETPDIRECT}
    mov SI,word ptr [NetData+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ELSE}
    mov SI,word ptr [buf+2]
    mov word ptr [SimInt.RealDS],SI
    mov word ptr [SimInt.RealSI],0
   {$ENDIF}
  {$ENDIF}
    push ds
    mov ah,$1D
    mov bx,[nd]
    mov cx,[len]
    mov dx,[flags]
   {$IFNDEF PMODE}
    lds si,[buf]
    les di,[fromaddr]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    mov [RetFlags],dx
    jc @retbad
    mov ax,cx
   {$IFDEF PMODE} {$IFDEF NETPDIRECT}
    LES DI,[buf]              {es:di points at dest (buf^)}
    CLD                       {make sure it is in the right direction}
    PUSH DS                   {save ds because we're gonna use it}
    mov ds,word ptr [NetData] {ds:si points to src (NetData^)}
    mov si,0                  {reg cx still has length}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF} {$ENDIF}
    jmp @done
   @retbad:
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {count or error returned in ax}
   @done:
end;


{***************************************************************}
{Status Calls}

{-------------------------------------------------}
{IsNetId checks nd to see if it is a valid net connection}
{Possible errors are: NOTNETCONN }
function IsNetnd(nd:integer):integer; assembler;  {ok}
asm
    push ds
    mov ah,$0D
    mov bx,[nd]
    CALL TCPsysInt
    pop ds
    jnc @ndisok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @ndisok:
    xor ax,ax
   @done:
end;

{-------------------------------------------------}
{NetInfo returns informtion on network interface for descriptor nd}
{Possible errors are: NOTNETCONN, NOHOST }
function NetInfo(nd:integer; ni:NetInfoPtr):integer; assembler; {ok}
asm
    push ds
    mov ah,$06
    mov bx,[nd]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nni+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[ni]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @niok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @niok:
    xor ax,ax              {result returned in ax}
   @done:
   {$IFDEF PMODE}
    LES DI,[ni]            {get dest ptr to es:di (ni^)}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV CX,NetInfoSize     {get size of record}
    MOV SI,0               {ds:si points to src (Dni^)}
    MOV DS,word ptr [Nni]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{returns input/ouput status of network descriptors}
{Possible errors are: BADARG }
function NetSelect(maxid:integer; iflags,oflags:pointer):integer; assembler; {ok}
asm
    push ds
    mov ah,$0E
    mov bx,[maxid]
   {$IFDEF PMODE}
    les di,[iflags]   {copy iflags to dtemp}
    mov si,es:[di]
    mov cx,es:[di+2]
    mov es,word ptr [Ntemp]
    mov di,0
    mov es:[di],si
    mov es:[di+2],cx
    les di,[oflags]   {copy oflags to dtemp+4}
    mov si,es:[di]
    mov cx,es:[di+2]
    mov es,word ptr [Ntemp]
    mov di,4
    mov es:[di],si
    mov es:[di+2],cx
    mov di,word ptr [Ntemp+2]        {point simint at dtemp}
    mov word ptr [SimInt.RealES],DI
    mov word ptr [SimInt.RealDS],DI
    mov word ptr [SimInt.RealDX],0   {iflags}
    mov word ptr [SimInt.RealDI],4   {oflags}
   {$ELSE}
    les di,[oflags]
    lds dx,[iflags]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
   {$IFDEF PMODE}
    mov es,word ptr [Ntemp]
    mov di,0
    mov si,es:[di]
    mov cx,es:[di+2]
    les di,[iflags]   {copy dtemp to iflags}
    mov es:[di],si
    mov es:[di+2],cx
    mov es,word ptr [Ntemp]
    mov di,4
    mov si,es:[di]
    mov cx,es:[di+2]
    les di,[oflags]   {copy dtemp+4 oflags}
    mov es:[di],si
    mov es:[di+2],cx
   {$ENDIF}
end;

{-------------------------------------------------}
{NetStat returns the statistics for the indicated descriptor}
{Possible errors are: NOTNETCONN }
function NetStat(nd:integer; stats:NetStatsPtr):integer; assembler; {ok}
asm
    push ds
    mov ah,$0C
    mov bx,[nd]
   {$IFDEF PMODE}
    MOV DX,word ptr [Nstats+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
   {$ELSE}
    lds dx,[stats]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @nstok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nstok:
    xor ax,ax              {result returned in AX}
   @done:
   {$IFDEF PMODE}
    LES DI,[stats]         {get dest ptr to es:di (stats^)}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV CX,NetStatsSize    {get size of record}
    MOV SI,0                 {ds:si points to src (Dstats^)}
    MOV DS,word ptr [Nstats]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;


{***************************************************************}
{Asynchronous Communications Calls}
{$IFNDEF PMODE}   {async calls are not available in pmode}

{-------------------------------------------------}
{Disables Asynch operation. Returns Z if was not enabled befor call}
{returns NZ if was enabled before call}
function DisableAsynch:integer; assembler;
asm
    push ds
    mov ah,$11
    CALL TCPsysInt
    pop ds
end;

{-------------------------------------------------}
{Enables Asynch operation. Returns Z if was enabled befor call}
{returns NZ if was disabled before call}
function EnableAsynch:integer; assembler;
asm
    push ds
    mov ah,$12
    CALL TCPsysInt
    pop ds
end;

{-------------------------------------------------}
{NetAsynch sets a handler for an asynch network call}
{Possible errors are: BADARG, NOTNETCONN }
function NetAsynch(fd,event:integer; handler:pointer; hint:longint):pointer; assembler;
asm
    push ds
    mov ah,$1F
    mov bx,[fd]
    lds dx,[handler]
    les di,[hint]
    CALL TCPsysInt
    pop ds
    mov dx,ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    mov dx,-1
   @done:
end;

{-------------------------------------------------}
{NetAlarm is an event trigger. nd is descriptor to which the alarm}
{is to be delivered. Atime is the number of milliseconds until the event}
{handler is the procedure to call at event time. hint is additional info}
{to be passed to the event handler. see NetAsync for more info}
{Possible errors are: NODESC, BADARG, REENTRY, BADSYSCALL }
function NetAlarm(fd:integer; Atime:longint;
                  handler:pointer; hint:longint):integer; assembler;
  asm
    push ds
    mov ah,$2B
    mov bx,[fd]
    mov dx,word ptr [Atime]
    mov cx,word ptr [Atime+2]
    lds si,[handler]
    les di,[hint]
    CALL TCPsysInt
    pop ds
    jnc @nalrmok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @nalrmok:
    xor ax,ax
   @done:
end;
{$ENDIF}

{***************************************************************}
{Host Name Resolution Calls}

{-------------------------------------------------}
{AdHtable returns the domain server name of the address passed in IPaddr}
{copies the name into the buffer pointed at by "name" "len" is the length}
{of that buffer. It must be large enough to hold the name. }
{255 charcters (string) or less. The name is null terminated.}
{Possible errors are: HOSTUNKNOWN, HOSTUNREACHABLE, NETUNREACHABLE }
{  NOSERVERS, PORTUNREACHABLE, PROTUNREACHABLE, SERVERERR, TIMEOUT }
function AdDomain(IPaddr:InName; name:pointer; len:word):integer; assembler; {ok}
asm
    push ds
    mov ah,$27
    mov bx,word ptr [IPaddr]
    mov dx,word ptr [IPaddr+2]
    mov cx,[len]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[name]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {AX returns result}
   @done:
   {$IFDEF PMODE}
    les di,[name]          {es:di points at dest (name^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dname^)}
    MOV DS,word ptr [Nname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{AdHtable returns the local host name of the address passed in IPaddr}
{copies the name into the buffer pointed at by "name" "len" is the length}
{of that buffer. It must be large enough to hold the name. }
{255 charcters (string) or less. The name is null terminated.}
{Possible errors are: NETDOSERR, HOSTUNKNOWN, CANTOPEN }
function AdHtable(IPaddr:InName; name:pointer; len:word):integer; assembler; {ok}
asm
    push ds
    mov ah,$26
    mov bx,word ptr [IPaddr]
    mov dx,word ptr [IPaddr+2]
    mov cx,[len]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[name]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1              {AX returns result}
   @done:
   {$IFDEF PMODE}
    les di,[name]          {es:di points at dest (name^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dname^)}
    MOV DS,word ptr [Nname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{AdResName returns the name of the address passed in IPaddr}
{copies the name into the buffer pointed at by "name" "len" is the length}
{of that buffer. It must be large enough to hold the name. }
{255 charcters (string) or less. The name is null terminated.}
{Possible errors are: HOSTUNKNOWN, HOSTUNREACHABLE, NETUNREACHABLE, }
{   NOSERVERS, PORTUNREACHABLE, PROTUNREACHABLE, SERVERERR, TIMEOUT }
function AdResName(IPaddr:InName; name:pointer; len:word):integer; assembler; {ok}
asm
    push ds
    mov ah,$25
    mov bx,word ptr [IPaddr]
    mov dx,word ptr [IPaddr+2]
    mov cx,[len]
   {$IFDEF PMODE}
    MOV SI,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[name]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
   {$IFDEF PMODE}
    les di,[name]          {es:di points at dest (name^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dname^)}
    MOV DS,word ptr [Nname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{NmHtable looks up name on host and returns the canonical name in cname}
{name must be null terminated. len is the length of the cname buf}
{returns fully qualified conical name in cname. name and cname }
{must be an array of char of 255 chars or less.}
{Possible errors are: HOSTUNKNOWN, HOSTUNREACHABLE, NETUNREACHABLE, }
{       NOSERVERS, PORTUNREACHABLE, PROTUNREACHABLE, SERVERERR, TIMEOUT}
function NmDomain(name,cname:pointer; len:integer):InName; assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NnameSize       {get size of record}
    mov es,word ptr [Nname]{es:di points to dest (Dname^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [name]
    MOV DS,word ptr [name+2] {ds:si points at src (name^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$52
   {$IFDEF PMODE}
    MOV DX,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
    MOV DI,word ptr [Ncname+2]
    MOV word ptr [SimInt.RealES],DI
    MOV word ptr [SimInt.RealDI],0
   {$ELSE}
    lds dx,[name]
    les di,[cname]
   {$ENDIF}
    mov cx,[len]
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    xor ax,ax
    xor dx,dx
   @done:
   {$IFDEF PMODE}
    les di,[cname]         {es:di points at dest (cname^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dcname^)}
    MOV DS,word ptr [Ncname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{NmHtable looks up name on host and returns the canonical name in cname}
{name must be null terminated. len is the length of the cname buf}
{returns fully qualified conical name in cname. name and cname }
{must be an array of char of 255 chars or less.}
{Possible errors are: NETDOSERR, HOSTUNKNOWN, NOSERVERS}
function NmHtable(name,cname:pointer; len:integer):InName; assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NnameSize       {get size of record}
    mov es,word ptr [Nname]{es:di points to dest (Dname^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [name]
    MOV DS,word ptr [name+2] {ds:si points at src (name^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$51
   {$IFDEF PMODE}
    MOV DX,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
    MOV DI,word ptr [Ncname+2]
    MOV word ptr [SimInt.RealES],DI
    MOV word ptr [SimInt.RealDI],0
   {$ELSE}
    lds dx,[name]
    les di,[cname]
   {$ENDIF}
    mov cx,[len]
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    xor ax,ax
    xor dx,dx
   @done:
   {$IFDEF PMODE}
    les di,[cname]         {es:di points at dest (cname^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dcname^)}
    MOV DS,word ptr [Ncname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{-------------------------------------------------}
{NmIen116 polls any ien116 servers for the address of name}
{Returns zero if failure, else returns InName of the name}
{name must be null terminated (pchar) or 255 chars or less.}
{Possible errors are: HOSTUNKNOWN, HOSTUNREACHABLE, NETUNREACHABLE,}
{              NOSERVERS, PORTUNREACHABLE, PROTUNREACHABLE, TIMEOUT}
function NmIen116(name:pointer):InName; assembler; {ok}
asm
  {$IFDEF PMODE}
    MOV CX,NnameSize       {get size of record}
    mov es,word ptr [Nname]{es:di points to dest (Dname^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [name]
    MOV DS,word ptr [name+2] {ds:si points at src (name^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$53
   {$IFDEF PMODE}
    MOV DX,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
   {$ELSE}
    lds dx,[name]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    xor ax,ax
    xor dx,dx
   @done:
end;

{-------------------------------------------------}
{NmPrsAddr parses name as an IP address}
{Returns zero if failure, else returns InName of the name}
{name must be null terminated (pchar).}
{Possible errors are: BADFORMAT }
function NmPrsAddr(name:pointer):InName; assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NnameSize       {get size of record}
    mov es,word ptr [Nname]{es:di points to dest (Dname^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [name]
    MOV DS,word ptr [name+2] {ds:si points at src (name^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$50
   {$IFDEF PMODE}
    MOV DX,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
   {$ELSE}
    lds dx,[name]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    xor ax,ax
    xor dx,dx
   @done:
end;

{-------------------------------------------------}
{NmResName resolves the host name provided in name }
{name must be null terminated. len is the length of the cname buf}
{returns fully qualified conical name in cname. name and cname }
{must be an array of char of 255 chars or less.}
{Possible errors are: NETDOSERR, HOSTUNKNOWN, HOSTUNREACHABLE, }
{                     NETUNREACHABLE, NOSERVERS, PORTUNREACHABLE, }
{                     SERVERERR, TIMEOUT }
function NmResName(name,cname:pointer; len:integer):InName; assembler; {ok}
asm
   {$IFDEF PMODE}
    MOV CX,NnameSize       {get size of record}
    mov es,word ptr [Nname]{es:di points to dest (Dname^)}
    mov di,0
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,word ptr [name]
    MOV DS,word ptr [name+2] {ds:si points at src (name^)}
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
    push ds
    mov ah,$54
   {$IFDEF PMODE}
    MOV DX,word ptr [Nname+2]
    MOV word ptr [SimInt.RealDS],DX
    MOV word ptr [SimInt.RealDX],0
    MOV DI,word ptr [Ncname+2]
    MOV word ptr [SimInt.RealES],DI
    MOV word ptr [SimInt.RealDI],0
   {$ELSE}
    lds dx,[name]
    les di,[cname]
   {$ENDIF}
    mov cx,[len]
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    xor ax,ax
    xor dx,dx
   @done:
   {$IFDEF PMODE}
    les di,[cname]         {es:di points at dest (cname^)}
    mov cx,[len]           {cx=length to copy}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV SI,0               {ds:si points to src (Dcname^)}
    MOV DS,word ptr [Ncname]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;

{***************************************************************}
{ICMP Calls}

{-------------------------------------------------}
{IcmpPing pings the indicated host}
{Possible errors are: HOSTUNREACHABLE, NETUNREACHABLE, SERVERERR, TIMEOUT}
function IcmpPing(Host:InName; Len:integer):integer; assembler;  {ok}
asm
    push ds
    mov ah,$30
    mov bx,word ptr [Host]
    mov dx,word ptr [Host+2]
    mov cx,[Len]
    CALL TCPsysInt
    pop ds
    jnc @done
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
   @done:
end;

{***************************************************************}
{Miscellaneous}

{-------------------------------------------------}
{GetNetVersion returns the current TCP driver version number}
{This function also saves the patch level in the variable 'patch'}
{Possible errors are: <none> }
function GetNetVersion:word; assembler;  {ok}
asm
    push ds
    mov ah,$0F
    CALL TCPsysInt
    pop ds
    mov [PatchLevel],bx
end;

{-------------------------------------------------}
{NetReconfig reconfigures the IP with structure}
{Possible errors are: <none> }
procedure NetReconfig; assembler;  {ok}
asm
    push ds
    mov ah,$17
    CALL TCPsysInt
    pop ds
end;

{-------------------------------------------------}
{NetShutDown releases the TCP/IP driver from memory}
{Possible errors are: <none> }
procedure NetShutDown; assembler;  {ok}
asm
    push ds
    mov ah,$10
    CALL TCPsysInt
    pop ds
end;

{-------------------------------------------------}
{Return the kernel configuration in record pointed at by kc}
{Possible errors are: REENTRY, BADSYSCALL }
function NetGetKConf(kc:KernelConfPtr):integer; assembler; {ok}
asm
    push ds
    mov ah,$2A
   {$IFDEF PMODE}
    MOV SI,word ptr [Nkc+2]
    MOV word ptr [SimInt.RealDS],SI
    MOV word ptr [SimInt.RealSI],0
   {$ELSE}
    lds si,[kc]
   {$ENDIF}
    CALL TCPsysInt
    pop ds
    jnc @gkcok
    mov [neterrno],al
    mov [netsuberrno],ah
    mov ax,-1
    jmp @done
   @gkcok:
    xor ax,ax
   @done:
   {$IFDEF PMODE}
    LES DI,[kc]            {get dest ptr to es:di (kc^)}
    CLD                    {make sure it is in the right direction}
    PUSH DS                {save ds because we're gonna use it}
    MOV CX,KernelConfSize  {get size of record}
    MOV SI,0               {ds:si points to src (Dkc^)}
    MOV DS,word ptr [Nkc]
    REP
    MOVSB                  {do the copy}
    POP DS                 {restore ds}
   {$ENDIF}
end;


{---------------------------------------------------}
{Byte Ordering}

{Convert Pascal longint to network longint}
function Htonl(L:longint):longint; {ok}
begin
  asm
     mov dx,word ptr [L]
     xchg dh,dl
     mov ax,word ptr [L]
     xchg ah,al
  end;
end;

{Convert Pascal word to network word order}
function Htons(W:word):word;
begin
  Htons := Swap(W);
end;

function Ntohl(L:longint):longint; {ok}
begin
  asm
     mov dx,word ptr [L]
     xchg dh,dl
     mov ax,word ptr [L]
     xchg ah,al
  end;
end;

function Ntohs(W:word):word; {ok}
begin
  Ntohs := Swap(W);
end;


{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

{Try to find the vector that the TCPTSR is hooked to}
{Returns 0 if vector not found - Returns the interrupt number if found }
function GetTcpTsrVector:integer;
  function IDmatch(v:byte):boolean;
  type Ctype = array[0..5] of char;
  var S : ^Ctype;
  begin
   {$IFNDEF PMODE}
    asm
      mov al,[v]    {getintvec via asm which avoids using dos unit}
      mov ah,$35    {this is the way the dos unit does it anyway}
      int $21       {except that it is more convoluted}
      add bx,3      {string occurs three bytes into code}
      mov word ptr [S+2],es
      mov word ptr [S],bx   {set string pointer to that location}
    end;
    if S^ = 'TCPTSR' then   {look for "TCPTSR" in the hooked TSR to}
      IDmatch := true       {see if it belongs to the PCTCP tsr}
    else                    {if it is there, return true, }
      IDmatch := false;     {else return false}
   {$ELSE}
    if v = PmodePcTcpInt then
      IDmatch := true
    else
      IDmatch := false;
   {$ENDIF}
  end;

var i : integer;
begin
  if PcTcpInt <> 0 then
  begin
    GetTcpTsrVector := PcTcpInt;  {We already know what the vector is}
    Exit;                         {so return that}
  end;
  for i := $20 to $E0 do          {search the available ints for the}
  begin                           {TCP TSR by looking for the id string}
    if IDmatch(i) then
    begin
      PcTcpInt := i;
      SetTCPvector(i);            {update our internal interrupt}
      GetTcpTsrVector := i;       {return the string if we find it}
      Exit;
    end;
  end;

  ZapTcpTsrCall;                  {nop out the tcp int call}
  GetTcpTsrVector := 0;           {return 0 if vector is not found}
end;

{---------------------------------------------------------------------}
{Abort exit procedure}

procedure PcTcpAbort;
var Result : integer;
begin
   Result := NetAbortAll;
end;

{---------------------}
{shut down procedures - do this before exiting to DOS}

var ExitSave:pointer;
procedure TcpSysExit;  far;
var Result : integer;
begin
   ExitProc := ExitSave;
   Result := NetReleaseAll;         {close down anything that is running}
 {$IFDEF PMODE}
   Nnap   := GlobalDosFree(loword(Nnap));   {release pointers}
   Nni    := GlobalDosFree(loword(Nni));
   Nstats := GlobalDosFree(loword(Nstats));
   Nkc    := GlobalDosFree(loword(Nkc));
   Nname  := GlobalDosFree(loword(Nname));
   Ncname := GlobalDosFree(loword(Nname));
   Ntemp  := GlobalDosFree(loword(Ntemp));
  {$IFDEF NETCOPY}
   NetData  := GlobalDosFree(loword(NetData));
  {$ENDIF}
 {$ENDIF}
end;


{------------}

begin
  {$IFDEF PMODE}
    Nnap  := GlobalDosAlloc(NetAddrSize);  {alloc working ptrs}
    Nni   := GlobalDosAlloc(NetInfoSize);
    Nstats:= GlobalDosAlloc(NetStatsSize);
    Nkc   := GlobalDosAlloc(KernelConfSize);
    Nname := GlobalDosAlloc(NnameSize);
    Ncname:= GlobalDosAlloc(NnameSize);
    Ntemp := GlobalDosAlloc(NtempSize);
   {$IFDEF NETCOPY}
    NetData := GlobalDosAlloc(NetDataSize);
   {$ENDIF}
  {$ENDIF}

   if GetTcpTsrVector <> 0 then
   begin
     ExitSave := ExitProc;
     ExitProc := @TcpSysExit;
     TcpInstalled := true;
   end
   else
   begin
     TcpInstalled := false;
     {if TCPtsr does not exist, nothing happens}
   end;
end.


